// -*- C++ -*-
/*!
 * @file  EV3Mindstorms.cpp
 * @brief EV3Mindstorms Move
 * @date $Date$
 *
 * $Id$
 */

#include "EV3Mindstorms.h"

// Module specification
// <rtc-template block="module_spec">
static const char* ev3mindstorms_spec[] =
  {
    "implementation_id", "EV3Mindstorms",
    "type_name",         "EV3Mindstorms",
    "description",       "EV3Mindstorms Move",
    "version",           "1.0.0",
    "vendor",            "Aizu",
    "category",          "EV3",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.scaling", "1.0",
    // Widget
    "conf.__widget__.scaling", "slider.0.1",
    // Constraints
    "conf.__constraints__.scaling", "0.1<=x<=1000.0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
EV3Mindstorms::EV3Mindstorms(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_PosIn("pos", m_Pos),
    m_VelIn("vel", m_Vel),
	motorA(OUTPUT_A),
	motorB(OUTPUT_B)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
EV3Mindstorms::~EV3Mindstorms()
{
}



RTC::ReturnCode_t EV3Mindstorms::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("pos", m_PosIn);
  addInPort("vel", m_VelIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("scaling", m_scaling, "1.0");
  // </rtc-template>
  
	velocityReceivedTimer = 0;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t EV3Mindstorms::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t EV3Mindstorms::onDeactivated(RTC::UniqueId ec_id)
{
	motorA.stop();
	motorB.stop();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t EV3Mindstorms::onExecute(RTC::UniqueId ec_id)
{
	/* 速度が更新されている場合 */
	if (m_VelIn.isNew())
	{
		/* 速度読込 */
		m_VelIn.read();
		
		int duty_cycle_sp_l = (int)((m_Vel.data.vx - 0.05925 * m_Vel.data.va) * m_scaling);
		int duty_cycle_sp_r = (int)((m_Vel.data.vx + 0.05925 * m_Vel.data.va) * m_scaling);

		/* 左回転割合が100を超えている場合 */
		if (duty_cycle_sp_l > 100)
		{
			duty_cycle_sp_l = 100;
		}
		/* 左回転割合が-100未満の場合 */
		else if (duty_cycle_sp_l < -100)
		{
			duty_cycle_sp_l = -100;
		}

		/* 右回転割合が100を超えている場合 */
		if (duty_cycle_sp_r > 100)
		{
			duty_cycle_sp_r = 100;
		}
		/* 右回転割合が-100未満の場合 */
		else if (duty_cycle_sp_r < -100)
		{
			duty_cycle_sp_r = -100;
		}

		cout<<"motor:"<<duty_cycle_sp_l<<","<<duty_cycle_sp_r<<endl;

		motorA.set_duty_cycle_sp(duty_cycle_sp_l);
		motorB.set_duty_cycle_sp(duty_cycle_sp_r);
		motorA.run_forever();
		motorB.run_forever();

		/* 速度取得時刻を設定 */
		time(&velocityReceivedTimer);
	}
	/* 位置情報が更新されている場合 */
	else if (m_PosIn.isNew())
	{
		time_t timer;

		/* 現在時刻を取得 */
		time(&timer);

		/* 現在時刻が最終速度取得時刻から1秒以上経過している場合 */
		if (timer > velocityReceivedTimer + 1)
		{
			/* 位置情報読込 */
			m_PosIn.read();

			int x = (int)(m_Pos.data[0] * m_scaling);
			int y = (int)(m_Pos.data[1] * m_scaling);

			int duty_cycle_sp_l;
			int duty_cycle_sp_r;

			/* 位置情報のｘ座標が0以上の場合 */
			if (x >= 0)
			{
				duty_cycle_sp_r = y;
				/* 位置情報のｙ座標が0以上の場合 */
				if (y >= 0)
				{
					duty_cycle_sp_l = (int)sqrt((double)(x * x + y * y));
				}
				else
				{
					duty_cycle_sp_l = (int)sqrt((double)(x * x + y * y)) * -1;
				}
			}
			else
			{
				duty_cycle_sp_l = y;
				/* 位置情報のｙ座標が0以上の場合 */
				if (y >= 0)
				{
					duty_cycle_sp_r = (int)sqrt((double)(x * x + y * y));
				}
				else
				{
					duty_cycle_sp_r = (int)sqrt((double)(x * x + y * y)) * -1;
				}
			}

			/* 左回転割合が100を超えている場合 */
			if (duty_cycle_sp_l > 100)
			{
				duty_cycle_sp_l = 100;
			}
			/* 左回転割合が-100未満の場合 */
			else if (duty_cycle_sp_l < -100)
			{
				duty_cycle_sp_l = -100;
			}

			/* 右回転割合が100を超えている場合 */
			if (duty_cycle_sp_r > 100)
			{
				duty_cycle_sp_r = 100;
			}
			/* 右回転割合が-100未満の場合 */
			else if (duty_cycle_sp_r < -100)
			{
				duty_cycle_sp_r = -100;
			}

			cout<<"motor:"<<duty_cycle_sp_l<<","<<duty_cycle_sp_r<<endl;

			motorA.set_duty_cycle_sp(duty_cycle_sp_l);
			motorB.set_duty_cycle_sp(duty_cycle_sp_r);
			motorA.run_forever();
			motorB.run_forever();
		}
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t EV3Mindstorms::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t EV3Mindstorms::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void EV3MindstormsInit(RTC::Manager* manager)
  {
    coil::Properties profile(ev3mindstorms_spec);
    manager->registerFactory(profile,
                             RTC::Create<EV3Mindstorms>,
                             RTC::Delete<EV3Mindstorms>);
  }
  
};


